@@ -12,7 +12,7 @@
 	if (fraction >= 1) {
 		graphics::lines(line, ...)
 	} else {
-		dist <- distHaversine(p[-nrow(p),], p[-1,]) * (1 - fraction)
+		dist <- distGeo(p[-nrow(p),], p[-1,]) * (1 - fraction)
 		bearing <- bearing(p[-nrow(p),], p[-1,])
 		p0 <- destPoint(p[-nrow(p),], bearing, dist)
 		for (i in 1:nrow(p0)) {
@@ -37,7 +37,7 @@
 plotArrows <- function(p, fraction=0.9, length=0.15, first='', add=FALSE, ...) {
 	asp=1
 	if (inherits(p, 'Spatial')) {
-		bb = t(bbox(p))
+		bb = t(sp::bbox(p))
 		interval = distm(bb)[2][1] / 1000
 		if (! add) { plot(bb, asp=asp, type='n') }
 		p = p@polygons
